"""
Testing for AGE-ABM Backend (backend, visual and utils)
@Author Max Hall
@Co-Authors Meghan Ireland and Matthew Fleischman

"""

from jsonpickle import dumps
import backend.backend as bk
import backend.backend_classes as bk_classes

class TestInitialisation:
    ''' Ensures that the class is of the correct type and initialises correctly '''
    def test__init__(self):
        ''' Ensures that the class is of the correct type and initialises correctly '''
        model = bk.ModelEnv("food")
        assert str(type(model)) == "<class 'backend.backend.ModelEnv'>"
        assert dumps(model.imports) == dumps({"ECAgent.Core" : bk_classes.ImportDetails("Core", "as"), "ECAgent.Tags" : bk_classes.ImportDetails("Tags", "as")
                        , "ECAgent.Collectors" : bk_classes.ImportDetails("Collectors", "as"), "numpy" : bk_classes.ImportDetails("np", "as")})
        assert dumps(model.components) == dumps({"Counter" : bk_classes.Component("This keeps count of number of agents", True
                                    , {"counter" : bk_classes.Attribute("int", "comp_lvl", 0)})})
        assert dumps(model.agents) == '{}'
        assert dumps(model.agents) == '{}'
        assert dumps({"model" : model.model_details}) == '{"model": {"py/object": "backend.backend_classes.ModelDetails", "description": "The Food Model", "name": "Food", "initial_parameters": {"num_agents": {}}, "gridworld": false, "sim_length": 1, "sim_seed": null}}'
        assert dumps(model.agents) == '{}'

        model = bk.ModelEnv("f#@od")
        assert str(type(model)) == "<class 'backend.backend.ModelEnv'>"
        assert dumps(model.imports) == dumps({"ECAgent.Core" : bk_classes.ImportDetails("Core", "as"), "ECAgent.Tags" : bk_classes.ImportDetails("Tags", "as")
                        , "ECAgent.Collectors" : bk_classes.ImportDetails("Collectors", "as"), "numpy" : bk_classes.ImportDetails("np", "as")})
        assert dumps(model.components) == dumps({"Counter" : bk_classes.Component("This keeps count of number of agents", True
                                    , {"counter" : bk_classes.Attribute("int", "comp_lvl", 0)})})
        assert dumps(model.agents) == '{}'
        assert dumps(model.agents) == '{}'
        assert dumps({"model" : model.model_details}) == '{"model": {"py/object": "backend.backend_classes.ModelDetails", "description": "The Default_Name Model", "name": "Default_Name", "initial_parameters": {"num_agents": {}}, "gridworld": false, "sim_length": 1, "sim_seed": null}}'
        assert dumps(model.agents) == '{}'



class TestImportMethods:
    ''' Tests for import methods '''
    def test_add_import(self):
        ''' Tests that the adding of imports works '''
        model = bk.ModelEnv("importtesting")
        response = model.add_import("ECAgent.More", "More", "as")
        assert response == 0
        response = model.add_import("ECAgent.Tomato", "Tom", "*")
        assert response == 0
        response = model.add_import("ECAgent.Cheese", "Che", "from", "penutbutter")
        assert response == 0
        response = model.add_import("ECAgent.More", "Lettuce", "as")
        assert response == 1
        response =  model.add_import("ECAgent.Lettuce", "Lettuce", "s")
        assert response == 2
        response = model.add_import("ECAgent.Lettuce", "Lettuce", "from")
        assert response == 3
        response = model.add_import("ECAgent.Lettuce", "Lettuce", "from", [])
        assert response == 3
        response = model.add_import("ECAgent.Lettuce", "Tom", "as")
        assert response == 4

    def test_edit_import(self):
        ''' Tests that the editing of imports works '''
        model = bk.ModelEnv("importtesting")
        response = model.edit_import("ECAgent.Core", nickname = "More")
        assert response == 0
        response = model.edit_import("ECAgent.Core", import_type = "*")
        assert response == 0
        response = model.edit_import("ECAgent.Core", import_type = "from", list_of_methods = ["cheese"])
        assert response == 0
        response = model.edit_import("ECAgent.Core", list_of_methods = ["tomatoes", "egg"])
        assert response == 0
        response = model.edit_import("ECAgent.Carrot", "car", "from", "penutbutter")
        assert response == 1
        response = model.edit_import("ECAgent.Core", nickname = "Tags")
        assert response == 2
        response =  model.edit_import("ECAgent.Core", import_type = "s")
        assert response == 3
        response = model.edit_import("ECAgent.Core", list_of_methods = [])
        assert response == 4
        response = model.edit_import("ECAgent.Core", import_type = "from", list_of_methods = [])
        assert response == 4
        response = model.edit_import("ECAgent.Core", list_of_methods = [])
        assert response == 4
        
        response = model.edit_import("ECAgent.Core", import_type = "*")
        assert response == 0
        response = model.edit_import("ECAgent.Core", list_of_methods = ["cheese"])
        assert response == 3



    def test_get_import(self):
        ''' Tests that the getting of imports works '''
        
        model = bk.ModelEnv("importtesting")
        response = dumps({"ECAgent.Core" : model.get_import("ECAgent.Core")})
        assert response == dumps({"ECAgent.Core" : bk_classes.ImportDetails("Core", "as")})
        response = model.get_import("ECAgent.Blue")
        assert response is None
        
    def test_remove_import(self):
        ''' Tests that the removing of imports works '''
        
        model = bk.ModelEnv("importtesting")
        response = model.remove_import("ECAgent.Core")
        assert response == 0
        model.edit_model_details(gridworld = True)
        response = model.remove_import("ECAgent.Environments")
        assert response == 0
        assert model.get_model_details().get_gridworld() is False
        response = model.remove_import("ECAgent.Blue")
        assert response == 1
        
    def test_get_list_imports(self):
        ''' Tests that the getting list of imports works '''
        
        model = bk.ModelEnv("importtesting")
        response = model.get_list_imports()
        assert set(response) == set(["ECAgent.Core", "ECAgent.Tags", "ECAgent.Collectors", "numpy"])
        model.imports = {}
        response = model.get_list_imports()
        assert len(response) == 0


class TestComponentMethods:
    ''' Tests for Component methods '''
    def test_add_component(self):
        ''' Tests that the adding of components works '''
        model = bk.ModelEnv("componenttesting")
        response = model.add_component("Comp1", "This is component")
        assert response == 0
        response = model.add_component("Comp2", "This is component", True)
        assert response == 0
        response = model.add_component("Comp 3", "This is component")
        assert response == 0
        response = model.add_component("Comp1", "This is component")
        assert response == 1
        response = model.add_component("Comp1", "This is component")
        assert response == 1
        response = model.add_component("Counter", "This is component")
        assert response == 1
        response = model.add_component("Com#p", "This is component")
        assert response == 2
        response = model.add_component("_Comp@", "This is component")
        assert response == 2

    def test_edit_component(self):
        ''' Tests that the editing of components works '''
        model = bk.ModelEnv("componenttesting")
        model.add_component("Comp1", "This is component for edit testing")
        model.add_component("Comp2", "This is component for edit testing")

        response = model.edit_component("Comp1", description = "Changed my mind")
        assert response == 0
        response = model.edit_component("Comp1", is_class_component = True)
        assert response == 0
        response = model.edit_component("Comp1", is_class_component = False)
        assert response == 0
        response = model.edit_component("Counter", description = "This is component")
        assert response == 1
        response = model.edit_component("Comp3", description = "This is component")
        assert response == 1
        
        # The following would return 2 if somehow this was in the model already, but add_component prevents it
        response = model.edit_component("Com#p")
        assert response == 1
        response = model.edit_component("_Comp@")
        assert response == 1

    def test_get_component(self):
        ''' Tests that the getting of components works '''
        
        model = bk.ModelEnv("componenttesting")
        model.add_component("Comp1", "This is component for get testing")

        response = dumps({"Comp1" : model.get_component("Comp1")})
        assert response == dumps({"Comp1" : bk_classes.Component("This is component for get testing", False, {})})
        response = model.get_component("Comp2")
        assert response is None


    def test_remove_component(self):
        ''' Tests that the removing of components works '''
        
        model = bk.ModelEnv("componenttesting")
        model.add_component("Comp1", "This is component for remove testing")
        model.add_attribute("Comp1", "attr1", "int", "model_lvl")

        response = model.remove_component("Comp1")
        assert response == 0
        response = model.remove_component("Comp1")
        assert response == 1
        
    def test_get_list_components(self):
        ''' Tests that the getting list of components works '''
        
        model = bk.ModelEnv("componenttesting")
        model.add_component("Comp1", "This is component")
        model.add_component("Comp2", "This is component", True)
        model.add_component("Comp 3", "This is component")

        # Tesing getting list if has values
        response = model.get_list_components()
        assert set(response) == set(["Comp1", "Comp2", "Comp_3"])
        # Tesing getting list if has no values
        model.components = {}
        response = model.get_list_components()
        assert len(response) == 0


class TestAttributeMethods:
    ''' Tests for Attribute methods '''
    def test_add_attribute(self):
        ''' Tests that the adding of attributes works '''
        model = bk.ModelEnv("attributetesting")
        model.add_component("Comp1", "This is component")
        model.add_component("CompClass1", "This is component")

        response = model.add_attribute("Comp1", "Attr1", "int", "comp_lvl", 1)
        assert response == 0
        response = model.add_attribute("Comp1", "Attr2", "bool", "comp_lvl", False)
        assert response == 0
        response = model.add_attribute("Comp1", "Attr3", "float", "comp_lvl", 0.1)
        assert response == 0
        response = model.add_attribute("Comp1", "Attr4", "int", "agent_lvl")
        assert response == 0
        response = model.add_attribute("Comp1", "Attr5", "bool", "agent_lvl")
        assert response == 0
        response = model.add_attribute("Comp1", "Attr6", "float", "agent_lvl")
        assert response == 0
        response = model.add_attribute("Comp1", "Attr7", "int", "model_lvl")
        assert response == 0
        response = model.add_attribute("Comp1", "Attr8", "bool", "model_lvl")
        assert response == 0
        response = model.add_attribute("Comp1", "Attr9", "float", "model_lvl")
        assert response == 0
        response = model.add_attribute("Comp1", "Attr 10", "bool", "model_lvl")
        assert response == 0
        response = model.add_attribute("CompClass1", "Attr1", "int", "model_lvl")
        assert response == 0
        response = model.add_attribute("CompClass1", "Attr2", "bool", "model_lvl")
        assert response == 0
        response = model.add_attribute("CompClass1", "Attr3", "float", "model_lvl")
        assert response == 0

        # This tests for error in attribute name
        response = model.add_attribute("Comp1", "Attr1", "bool", "comp_lvl", False)
        assert response == 1
        response = model.add_attribute("Comp1", "Attr1!@", "bool", "comp_lvl", False)
        assert response == 2

        # This tests for error in component name
        response = model.add_attribute("Comp21", "Attr11", "bool", "comp_lvl", False)
        assert response == 1
        # This would return 2 if the component was already invalid in the system, but that is prevented already
        response = model.add_attribute("Comp1@", "Attr11", "bool", "comp_lvl", False)
        assert response == 1

        # Tests for invalid type attribute
        response = model.add_attribute("CompClass1", "Attr4", "flat", "model_lvl")
        assert response == 3

        # This tests for comp_level without default value
        response = model.add_attribute("Comp1", "Attr11", "int", "comp_lvl")
        assert response == 4

        # This tests for invalid define level name
        response = model.add_attribute("Comp1", "Attr11", "float", "comppppp_lvl", 0.1)
        assert response == 5


    def test_edit_attribute(self):
        ''' Tests that the editing of attributes works '''
        model = bk.ModelEnv("attributetesting")
        model.add_component("Comp1", "This is component for edit testing")
        model.add_component("Comp2", "This is component for edit testing")
        model.add_attribute("Comp1", "Attr1", "int", "comp_lvl", 1)
        model.add_attribute("Comp1", "Attr2", "int", "model_lvl")
        model.add_attribute("Comp1", "Attr3", "int", "model_lvl")


        response = model.edit_attribute("Comp1", "Attr1", type_attr = "bool", default_value = True)
        assert response == 0
        response = model.edit_attribute("Comp1", "Attr1", define_level = "model_lvl")
        assert response == 0
        response = model.edit_attribute("Comp1", "Attr3", define_level = "agent_lvl", default_value = [1234,1])
        assert response == 0
        response = model.edit_attribute("Comp1", "Attr1", default_value = False)
        assert response == 0

        # This tests for error in attribute name
        response = model.edit_attribute("Comp1", "Attr4", type_attr = "bool", default_value = True)
        assert response == 1
        response = model.edit_attribute("Comp2", "Attr@1", type_attr = "bool", default_value = True)
        assert response == 1
        response = model.edit_attribute("Comp1", "Attr#", type_attr = "bool", default_value = True)
        assert response == 1
        
        # This tests for error in component name
        response = model.edit_attribute("Comp21", "Attr1", type_attr = "bool", default_value = True)
        assert response == 1
        # This would return 2 if the component was already invalid in the system, but that is prevented already
        response = model.edit_attribute("Comp#@1", "Attr1", type_attr = "bool", default_value = True)
        assert response == 1
        
        # Wrong input for type_attr and define_level
        response = model.edit_attribute("Comp1", "Attr1", type_attr = "bol", default_value = True)
        assert response == 3
        response = model.edit_attribute("Comp1", "Attr1", define_level = "modelll_lvl")
        assert response == 4

        # If define level is component, but no default value provided
        response = model.edit_attribute("Comp1", "Attr1", define_level = "comp_lvl")
        assert response == 5


    def test_get_attribute(self):
        ''' Tests that the getting of attributes works '''
        
        model = bk.ModelEnv("attributetesting")
        model.add_component("Comp1", "This is component for get testing")
        model.add_component("Comp2", "This is component for get testing")
        model.add_attribute("Comp1", "Attr1", "int", "comp_lvl", 1)
        model.add_attribute("Comp1", "Attr2", "int", "model_lvl")

        response = dumps({"Attr1" : model.get_attribute("Comp1", "Attr1")})
        assert response == dumps({"Attr1" : bk_classes.Attribute("int", "comp_lvl", 1)})
        response = model.get_attribute("Comp1", "Attr4")
        assert response is None
        response = model.get_attribute("Comp2", "Attr1")
        assert response is None
        response = model.get_attribute("Comp3", "Attr1")
        assert response is None


    def test_remove_attribute(self):
        ''' Tests that the removing of attributes works '''
        
        model = bk.ModelEnv("attributetesting")
        model.add_component("Comp1", "This is component for remove testing")
        model.add_component("Comp2", "This is component for remove testing")
        model.add_attribute("Comp1", "Attr1", "int", "comp_lvl", 1)
        model.add_attribute("Comp1", "Attr2", "int", "model_lvl")

        response = model.remove_attribute("Comp1", "Attr1")
        assert response == 0
        response = model.remove_attribute("Comp1", "Attr2")
        assert response == 0
        response = model.remove_attribute("Comp1", "Attr1")
        assert response == 1
        response = model.remove_attribute("Comp2", "Attr1")
        assert response == 1
        response = model.remove_attribute("Comp3", "Attr1")
        assert response == 1


    def test_get_list_attributes(self):
        ''' Tests that the getting list of attributes works '''
        
        model = bk.ModelEnv("attributetesting")
        model.add_component("Comp1", "This is component for get list attributes testing")
        model.add_component("Comp2", "This is component for get list attributes testing")
        model.add_attribute("Comp1", "Attr1", "int", "comp_lvl", 1)
        model.add_attribute("Comp1", "Attr2", "int", "model_lvl")

        # Tesing getting list if has values
        response = model.get_list_attributes("Comp1")
        assert set(response) == set(["attr1", "attr2"])
        
        # Tesing getting list if has no values
        response = model.get_list_attributes("Comp2")
        assert set(response) == set()
        
        # Tesing getting list if invalid name
        response = model.get_list_attributes("Comp3")
        assert set(response) == set()


class TestAgentMethods:
    ''' Tests for Agent methods '''
    def test_add_agent(self):
        ''' Tests that the adding of agents works '''
        model = bk.ModelEnv("agenttesting")
        model.add_component("Comp1", "This is component for add testing")
        model.add_component("Comp2", "This is component for add testing")

        response = model.add_agent("Agent1", "This is agent", ["Comp1", "Comp2"])
        assert response == 0
        response = model.add_agent("Agent 2", "This is agent", ["Comp1"])
        assert response == 0
        response = model.add_agent("Agent ", "This is agent", [])
        assert response == 0

        # Adding agent initial co-ords if gridworld
        model.get_model_details().edit_gridworld(True)
        response = model.add_agent("Agent3", "This is agent", ["Comp1", "Comp2"], [1,1])
        assert response == 0
        response = model.add_agent("Agent4", "This is agent", ["Comp1", "Comp2"])
        assert response == 4

        model.get_model_details().edit_gridworld(False)
        # Name errors
        response = model.add_agent("Agent1", "This is agent", ["Comp1"])
        assert response == 1
        response = model.add_agent("3Agent1", "This is agent", [])
        assert response == 2
        response = model.add_agent("Ag@ent1", "This is agent", [])
        assert response == 2

        # Component list errors
        response = model.add_agent("Agent4", "This is agent", ["Comp3"])
        assert response == 3
        response = model.add_agent("Agent4", "This is agent", ["Comp1", "Comp5"])
        assert response == 3
        response = model.add_agent("Agent4", "This is agent", ["#Comp1"])
        assert response == 3


    def test_edit_agent(self):
        ''' Tests that the editing of agents works '''
        model = bk.ModelEnv("agenttesting")
        model.add_component("Comp1", "This is component for edit testing")
        model.add_component("Comp2", "This is component for edit testing")
        model.add_component("Comp3", "This is component for edit testing", True)
        model.add_attribute("Comp3", "attr1", "int", "model_lvl")
        model.add_agent("Agent1", "This is agent", ["Comp1", "Comp2", "Comp3"])
        model.add_agent("Agent2", "This is agent", ["Comp1"])

        response = model.edit_agent("Agent1", description = "Changed my mind")
        assert response == 0
        response = model.edit_agent("agent1", description = "Changed my mind again!")
        assert response == 0
        response = model.edit_agent("Agent1", given_components = ["Comp1"])
        assert response == 0
        response = model.edit_agent("Agent1", given_components = ["Comp1", "Comp2", "Comp3"])
        assert response == 0

        # Name errors
        response = model.edit_agent("Agent3", description = "This is agent")
        assert response == 1
        response = model.edit_agent("Agen@t2", given_components = ["Comp1"])
        assert response == 1
        # The following would return 2 if somehow this was in the model already, but add_agent prevents it
        response = model.edit_agent("Agen#t")
        assert response == 1
        response = model.edit_agent("_Agent@")
        assert response == 1

        # Component name errors
        response = model.edit_agent("Agent1", given_components = ["Comp4"])
        assert response == 3
        response = model.edit_agent("Agent1", given_components = ["Comp1", "Comp5"])
        assert response == 3
        response = model.edit_agent("Agent2", given_components = ["#Comp1"])
        assert response == 3

        # If gridworld
        model.get_model_details().edit_gridworld(True)
        response = model.edit_agent("agent1", description = "Changed my mind again!", init_coordinates = [1,2])
        assert response == 0
        response = model.edit_agent("agent2", description = "Changed my mind again again!!")
        assert response == 4


    def test_get_agent(self):
        ''' Tests that the getting of agents works '''
        
        model = bk.ModelEnv("agenttesting")
        model.add_component("Comp1", "This is component for get testing")
        model.add_component("Comp2", "This is component for get testing")
        response = model.add_agent("Agent1", "This is agent", ["Comp1", "Comp2"])

        response = dumps({"Agent1" : model.get_agent("Agent1")})
        assert response == dumps({"Agent1" : bk_classes.Agent("This is agent", ["Comp1", "Comp2", "Counter"])})
        response = model.get_agent("Agent2")
        assert response is None


    def test_remove_agent(self):
        ''' Tests that the removing of agents works '''
        
        model = bk.ModelEnv("agenttesting")
        model.add_component("Comp1", "This is component for remove testing")
        model.add_component("Comp2", "This is component for remove testing")
        model.add_component("Comp3", "This is component for edit testing", True)
        model.add_attribute("Comp3", "attr1", "int", "model_lvl")
        model.add_agent("Agent1", "This is agent", ["Comp1", "Comp2", "Comp3"])

        response = model.remove_agent("Agent1")
        assert response == 0
        response = model.remove_agent("Agent1")
        assert response == 1
        
    def test_get_list_agents(self):
        ''' Tests that the getting list of agents works '''
        
        model = bk.ModelEnv("agenttesting")
        model.add_component("Comp1", "This is component for list testing")
        model.add_component("Comp2", "This is component for list testing")
        model.add_agent("Agent1", "This is agent", ["Comp1", "Comp2"])
        model.add_agent("Agent2", "This is agent", ["Comp1"])
        model.add_agent("Agent3", "This is agent", ["Comp2"])

        # Tesing getting list if has values
        response = model.get_list_agents()
        assert set(response) == set(["Agent1", "Agent2", "Agent3"])
        # Tesing getting list if has no values
        model.agents = {}
        response = model.get_list_agents()
        assert len(response) == 0


    def test_add_agent_lvl_default_value(self):
        ''' Tests that setting agent level default values of agents works '''
        model = bk.ModelEnv("agenttesting")
        model.add_component("Comp1", "This is component for remove testing")
        model.add_component("Comp2", "This is component for remove testing")
        model.add_attribute("Comp1", "Attr1", "int", "agent_lvl")
        model.add_attribute("Comp1", "Attr2", "bool", "agent_lvl")
        model.add_attribute("Comp2", "Attr1", "int", "comp_lvl", 1)
        model.add_attribute("Comp2", "Attr2", "float", "model_lvl")
        model.add_agent("Agent1", "This is agent", ["Comp1", "Comp2"])
        model.add_agent("Agent2", "This is agent", ["Comp1"])

        response = model.add_agent_lvl_default_value("Agent1", "Comp1", "attr1", "1 + 1")
        assert response == 0
        response = model.add_agent_lvl_default_value("agent1", "Comp1", "attr2", "1 + 1")
        assert response == 0
        response = model.add_agent_lvl_default_value("Agent2", "Comp1", "Attr1", "1 + 1")
        assert response == 0
        response = model.add_agent_lvl_default_value("agent2", "comp1", "Attr2", "1 + 1")
        assert response == 0

        # Name errors
        response = model.add_agent_lvl_default_value("Agent 1", " C 1", "attr 1", "1 + 1")
        assert response == 1
        response = model.add_agent_lvl_default_value("agen$1", "Comp 1", "attr2", "1 + 1")
        assert response == 1
        response = model.add_agent_lvl_default_value("Age2", "Comp 1", "Ar1", "1 + 1")
        assert response == 1
        response = model.add_agent_lvl_default_value("agent2", "comp 1", "A#tr2", "1 + 1")
        assert response == 1
        response = model.add_agent_lvl_default_value("agent2", "comp1", "Attr3", "1 + 1")
        assert response == 1

        
        # Testing with attributes that aren't agent level defined
        response = model.add_agent_lvl_default_value("Agent1", "comp2", "Attr1", "1 + 1")
        assert response == 3
        response = model.add_agent_lvl_default_value("Agent1", "Comp2", "attr2", "1 + 1")
        assert response == 3


class TestSystemMethods:
    ''' Tests for System methods '''
    def test_add_system(self):
        ''' Tests that the adding of systems works '''
        model = bk.ModelEnv("systemtesting")

        response = model.add_system("System1", "This is system")
        assert response == 0
        response = model.add_system("system 2", "This is system", start = 5, end = 10, frequency = 2)
        assert response == 0
        response = model.add_system("System 3", "This is system", generators = {"gen1" : "genObj"})
        assert response == 0

        # Name errors
        response = model.add_system("system1", "This is system")
        assert response == 1
        response = model.add_system("System@1", "This is system", start = 5, end = 10, frequency = 2)
        assert response == 2
        response = model.add_system("^System 3", "This is system")
        assert response == 2


    def test_edit_system(self):
        ''' Tests that the editing of systems works '''
        model = bk.ModelEnv("systemtesting")
        model.add_system("System1", "This is system")
        model.add_system("System2", "This is system", start = 5, end = 10, frequency = 2)


        response = model.edit_system("System1", description = "Changed my mind")
        assert response == 0
        response = model.edit_system("System2", start = 50, end = 100, frequency = 20)
        assert response == 0
        response = model.edit_system("System2", end = 100)
        assert response == 0
        response = model.edit_system("System2", execute_function_body = "\tdef execute():\n\t\tpass")
        assert response == 0

        # Name errors
        response = model.edit_system("System3", description = "This is system")
        assert response == 1
        response = model.edit_system("System#3", start = 10)
        assert response == 1
        # The following would return 2 if somehow this was in the model already, but add_system prevents it
        response = model.edit_system("Sys#tem")
        assert response == 1
        response = model.edit_system("S^ystem@")
        assert response == 1


    def test_get_system(self):
        ''' Tests that the getting of systems works '''
        
        model = bk.ModelEnv("systemtesting")
        model.add_system("System1", "This is system for get testing", start = 2, end = 10, frequency = 200)

        response = dumps({"System1" : model.get_system("System1")})
        assert response == dumps({"System1" : bk_classes.System("This is system for get testing", start = 2, end = 10, frequency = 200)})
        response = model.get_system("System2")
        assert response is None


    def test_remove_system(self):
        ''' Tests that the removing of systems works '''
        
        model = bk.ModelEnv("systemtesting")
        model.add_system("System1", "This is system for remove testing", start = 2, end = 10, frequency = 200)

        response = model.remove_system("System1")
        assert response == 0
        response = model.remove_system("System1")
        assert response == 1

     
    def test_get_list_systems(self):
        ''' Tests that the getting list of systems works '''
        
        model = bk.ModelEnv("systemtesting")
        model.add_system("System1", "This is system for get list testing", start = 2, end = 10, frequency = 200)
        model.add_system("system 2", "This is system for get list testing", start = 2, frequency = 200)
        model.add_system("System 3", "This is system for get list testing", frequency = 200)
        model.add_system("system4", "This is system for get list testing")

        # Tesing getting list if has values
        response = model.get_list_systems()
        assert set(response) == set(["System1", "System_2", "System_3", "System4"])
        # Tesing getting list if has no values
        model.systems = {}
        response = model.get_list_systems()
        assert len(response) == 0


class TestGeneratorMethods:
    ''' Tests for Generator methods '''
    def test_add_generator(self):
        ''' Tests that the adding of generators works '''
        model = bk.ModelEnv("generatortesting")
        model.add_system("System1", "This is system")
        model.add_system("System2", "This is system")

        # relevent_system : str, name : str, description : str, nickname : str = None, return_value : str = None
        response = model.add_generator("System1", "generator1", "This is generator", nickname = "gen1", return_value = "return 5 if cool else 2")
        assert response == 0
        response = model.add_generator("System1", "generator2", "This is generator", return_value = "return 5 if cool else 2")
        assert response == 0
        response = model.add_generator("System2", "generator2", "This is generator", nickname = "gen2", return_value = "return 1 if cool else 0.5")
        assert response == 0
        response = model.add_generator("System2", "generator3", "This is generator", nickname = "gen3", return_value = "return 2 if cool else 5")
        assert response == 0

        # Name errors
        response = model.add_generator("System1", "generator1", "This is generator", nickname = "gen1", return_value = "return 5 if cool else 2")
        assert response == 1
        response = model.add_generator("System3", "generator1", "This is generator", nickname = "gen5", return_value = "return 5 if cool else 2")
        assert response == 1
        response = model.add_generator("System1", "generator^*&1", "This is generator", nickname = "gen1", return_value = "return 5 if cool else 2")
        assert response == 2

        # nickname already taken
        response = model.add_generator("System1", "generator3", "This is generator", nickname = "gen1", return_value = "return 5 if cool else 2")
        assert response == 3


    def test_edit_generator(self):
        ''' Tests that the editing of generators works '''
        model = bk.ModelEnv("generatortesting")
        model.add_system("System1", "This is system")
        model.add_system("System2", "This is system")
        model.add_generator("System1", "generator1", "This is generator", nickname = "gen1", return_value = "return 5 if cool else 2")
        model.add_generator("System1", "generator2", "This is generator", nickname = "gen2", return_value = "return 3 if cool else 2")


        response = model.edit_generator("System1", "Generator1", description = "Changed my mind")
        assert response == 0
        response = model.edit_generator("System1", "Generator1", return_value = "return 5")
        assert response == 0
        response = model.edit_generator("System1", "Generator1", nickname = "generator25")
        assert response == 0

        # Name errors
        response = model.edit_generator("System2", "Gene53", nickname = "gen12")
        assert response == 1
        response = model.edit_generator("System4", "genene5", nickname = "gen12")
        assert response == 1
        # The following would return 2 if somehow this was in the model already, but add_generator prevents it
        response = model.edit_generator("System2", "afsr@4d", nickname = "gen12")
        assert response == 1
        response = model.edit_generator("System2", "faeafw#@", nickname = "gen2")
        assert response == 1

        # Cannot change a nickname to an existing one
        response = model.edit_generator("System1", "Generator1", nickname = "gen2")
        assert response == 3


    def test_get_generator(self):
        ''' Tests that the getting of generators works '''
        
        model = bk.ModelEnv("generatortesting")
        model.add_system("System1", "This is system")
        model.add_generator("System1", "generator1", "This is generator", nickname = "gen1", return_value = "return 5 if cool else 2")

        response = dumps({"System1" : model.get_generator("System1", "Generator1")})
        assert response == dumps({"System1" : bk_classes.Generator("This is generator", "gen1", "return 5 if cool else 2", {})})
        response = model.get_generator("System2", "Generator1")
        assert response is None
        response = model.get_generator("System1", "Generator2")
        assert response is None


    def test_remove_generator(self):
        ''' Tests that the removing of generators works '''
        
        model = bk.ModelEnv("generatortesting")
        model.add_system("System1", "This is system")
        model.add_generator("System1", "generator1", "This is generator", nickname = "gen1", return_value = "return 5 if cool else 2")

        response = model.remove_generator("System1", "Generator1")
        assert response == 0
        response = model.remove_generator("System1", "Generator1")
        assert response == 1
        
        # Case of System name error
        response = model.remove_system("System1")
        response = model.remove_generator("System 1", "Generator1")
        assert response == 1

        # Case of generator name error
        model.add_system("System1", "This is system")
        model.add_generator("System1", "generator1", "This is generator", nickname = "gen1", return_value = "return 5 if cool else 2")
        response = model.remove_generator("System1", "Generator2")
        assert response == 1

     
    def test_get_list_generators(self):
        ''' Tests that the getting list of generators works '''
        
        model = bk.ModelEnv("generatortesting")
        model.add_system("System1", "This is system")
        model.add_system("System2", "This is system")
        model.add_generator("System1", "generator1", "This is generator", nickname = "gen1", return_value = "return 5 if cool else 2")
        model.add_generator("System1", "generator2", "This is generator", nickname = "gen2", return_value = "return 3 if cool else 2")
        model.add_generator("System1", "generator3", "This is generator", nickname = "gen3", return_value = "return 3 if cool else 2")
        model.add_generator("System2", "generator1", "This is generator", nickname = "gen1", return_value = "return 5 if cool else 2")
        model.add_generator("System2", "generator2", "This is generator", nickname = "gen2", return_value = "return 3 if cool else 2")

        # Tesing getting list if has values
        response = model.get_list_generators("System1")
        assert set(response) == set(["Generator1", "Generator2", "Generator3"])
        response = model.get_list_generators("System2")
        assert set(response) == set(["Generator1", "Generator2"])
        # Tesing getting list if has no values
        model.generators = {}
        response = model.get_list_generators("System4")
        assert len(response) == 0


class TestGenAttributeMethods:
    ''' Tests for Generator Attribute methods '''
    def test_add_gen_attribute(self):
        ''' Tests that the adding of generator attributes works '''
        model = bk.ModelEnv("generatortesting")
        model.add_system("System1", "This is system")
        model.add_generator("System1", "Generator1", "This is generator", nickname = "gen1", return_value = "return 5 if cool else 2")
        model.add_generator("System1", "Generator2", "This is generator", nickname = "gen2", return_value = "return 3 if cool else 2")

        # relevent_system : str, name : str, description : str, nickname : str = None, return_value : str = None
        response = model.add_gen_attribute("System1", "Generator1", "Gen Att 1", "int")
        assert response == 0
        response = model.add_gen_attribute("System1", "Generator1", "Gen Att 2", "float")
        assert response == 0
        response = model.add_gen_attribute("System1", "Generator2", "Gen Att 3", "bool")
        assert response == 0

        # Name errors
        response = model.add_gen_attribute("System1", "Generator1", "Gen Att 1", "int")
        assert response == 1
        response = model.add_gen_attribute("System1", "Generator 1", "Gen Att 1", "int")
        assert response == 1
        response = model.add_gen_attribute("System 1", "Generator1", "Gen Att 1", "int")
        assert response == 1
        response = model.add_gen_attribute("System1", "Generator2", "Gen Att@1", "int")
        assert response == 2

        # Invalid attr_type
        response = model.add_gen_attribute("System1", "Generator2", "Gen Att 5", "in")
        assert response == 3


    def test_get_gen_attribute(self):
        ''' Tests that the getting of generator attributes works '''
        
        model = bk.ModelEnv("generatortesting")
        model.add_system("System1", "This is system")
        model.add_generator("System1", "generator1", "This is generator", nickname = "gen1", return_value = "return 5 if cool else 2")
        model.add_gen_attribute("System1", "Generator1", "Gen Att 1", "int")

        response = dumps({"Gen_Att_1" : model.get_gen_attribute("System1", "Generator1", "Gen_Att_1")})
        assert response == dumps({"Gen_Att_1" : bk_classes.Attribute("int", "model_lvl")})
        response = model.get_gen_attribute("System2", "Generator1", "gen_attr_1")
        assert response is None
        response = model.get_gen_attribute("System1", "Generator2", "gen_attr_1")
        assert response is None
        response = model.get_gen_attribute("System1", "Generator1", "gen_attr_2")
        assert response is None


    def test_remove_gen_attribute(self):
        ''' Tests that the removing of generators works '''
        
        model = bk.ModelEnv("generatortesting")
        model.add_system("System1", "This is system")
        model.add_generator("System1", "generator1", "This is generator", nickname = "gen1", return_value = "return 5 if cool else 2")
        model.add_gen_attribute("System1", "Generator1", "Gen Att 1", "int")

        response = model.remove_gen_attribute("System1", "Generator1", "Gen_Att_1")
        assert response == 0
        response = model.remove_gen_attribute("System1", "Generator1", "Gen_Att_1")
        assert response == 1

        # Case of System name error
        model.remove_system("System1")
        response = model.remove_gen_attribute("System 1", "Generator1", "Gen_Att_1")
        assert response == 1

        # Case of generator, then attribute name error
        model.add_system("System1", "This is system")
        model.add_generator("System1", "generator1", "This is generator", nickname = "gen1", return_value = "return 5 if cool else 2")
        model.add_gen_attribute("System1", "Generator1", "Gen Att 1", "int")
        response = model.remove_gen_attribute("System1", "Generator2", "Gen_Att_1")
        assert response == 1
        response = model.remove_gen_attribute("System1", "Generator1", "Gen_Att_2")
        assert response == 1

     
    def test_get_list_gen_attributes(self):
        ''' Tests that the getting list of generators works '''
        
        model = bk.ModelEnv("generatortesting")
        model.add_system("System1", "This is system")
        model.add_generator("System1", "generator1", "This is generator", nickname = "gen1", return_value = "return 5 if cool else 2")
        model.add_generator("System1", "generator2", "This is generator", nickname = "gen2", return_value = "return 3 if cool else 2")
        model.add_gen_attribute("System1", "Generator1", "Gen Att 1", "int")
        model.add_gen_attribute("System1", "Generator1", "Gen Att 2", "int")
        model.add_gen_attribute("System1", "Generator1", "Gen Att 3", "int")
        model.add_gen_attribute("System1", "Generator1", "Gen Att 4", "int")
        model.add_gen_attribute("System1", "Generator2", "Gen Att 1", "int")
        model.add_gen_attribute("System1", "Generator2", "Gen Att 2", "int")


        # Tesing getting list if has values
        response = model.get_list_gen_attributes("System1", "generator1")
        assert set(response) == set(["gen_att_1", "gen_att_2", "gen_att_3", "gen_att_4"])
        response = model.get_list_gen_attributes("System1", "generator2")
        assert set(response) == set(["gen_att_1", "gen_att_2"])
       
        # Testing if invalid name:
        response = model.get_list_gen_attributes("System3", "generator1")
        assert set(response) == set()
        response = model.get_list_gen_attributes("System1", "generator3")
        assert set(response) == set()
       
        # Tesing getting list if has no values
        model.get_generator("System1", "generator1").attributes = {}
        response = model.get_list_gen_attributes("System1", "generator1")
        assert len(response) == 0


class TestModelDetailsMethods:
    ''' Tests for Model Details methods '''
    def test_edit_model_details(self):
        ''' Tests that the editing of model works '''
        model = bk.ModelEnv("modeltesting")

        response = model.edit_model_details(name = "Cool new Thing", description = "new cool description", gridworld = True)
        assert response == 0
        response = model.edit_model_details(sim_length = 1000, sim_seed = "20")
        assert response == 0

        # Name errors
        response = model.edit_model_details(name = "Cool new thing")
        assert response == 1
        response = model.edit_model_details(name = "$#Hey Bro")
        assert response == 2

    def test_get_model_details(self):
        ''' Tests that the get of model works '''
        model = bk.ModelEnv("modeltesting")

        response = dumps({"Model_Details" : model.get_model_details()})
        assert response == dumps({"Model_Details" : bk_classes.ModelDetails("Modeltesting", f"The Modeltesting Model")})


class TestExecuteMethods:
    ''' Tests for Execute methods '''
    def test_add_execute(self):
        ''' Tests that the add of execute works '''
        model = bk.ModelEnv("executetesting")
        model.add_system("System1", "This is my system")

        response = model.add_execute("System1","\tdef execute():\n\t\tpass")
        assert response == 0
        
        # Name errors
        response = model.add_execute("Sys#@@@tem1","\tdef execute():\n\t\tpass")
        assert response == 1
        response = model.add_execute("Sys3tem1","\tdef execute():\n\t\tpass")
        assert response == 1


    def test_get_execute(self):
        ''' Tests that the get of execute works '''
        model = bk.ModelEnv("executetesting")
        model.add_system("System1", "This is my system")

        response = model.get_execute("System1")
        assert response == "\tdef execute(self):\n\t\t# Do not edit above this line!\n\t\t# Code below and indent correctly using tabs, not spaces!\n\t\tpass"

        # Name errors
        response = model.get_execute("Sys#@@@tem1")
        assert response == None
        response = model.get_execute("Sys3tem1")
        assert response == None


class TestAgentOfInterestMethods:
    ''' Tests for Agent Of Interest methods '''
    def test_add_agent_of_interest(self):
        ''' Tests that the add of execute works '''
        model = bk.ModelEnv("agentofinteresttesting")
        model.add_component("Comp1", "This is component for add testing")
        model.add_component("Comp2", "This is component for add testing")
        model.add_agent("Agent1", "This is agent", ["Comp1", "Comp2"])
        model.add_agent("Agent2", "This is agent", ["Comp1", "Comp2"])
        model.add_agent("Agent3", "This is agent", ["Comp1"])
        model.add_agent("Agent4", "This is agent", ["Comp1"])


        response = model.add_agent_of_interest("Agent1", ["Comp1"])
        assert response == 0
        response = model.add_agent_of_interest("Agent2", ["Comp1", "Comp2"])
        assert response == 0
        response = model.add_agent_of_interest("Agent3", ["Comp1"])
        assert response == 0
    
        # Name errors
        response = model.add_agent_of_interest("Agent1", ["Comp1"])
        assert response == 1
        response = model.add_agent_of_interest("Agent5", ["Comp 1"])
        assert response == 1
        response = model.add_agent_of_interest("Agent4", ["Comp 1", "Comp2"])
        assert response == 1
        response = model.add_agent_of_interest("Agen@3", ["Comp 1"])
        assert response == 2

        # No agents of interest provided
        response = model.add_agent_of_interest("Agent4")
        assert response == 3


    def test_get_agent_of_interest(self):
        ''' Tests that the get of execute works '''
        model = bk.ModelEnv("agentofinteresttesting")
        model.add_component("Comp1", "This is component for add testing")
        model.add_component("Comp2", "This is component for add testing")
        model.add_agent("Agent1", "This is agent", ["Comp1", "Comp2"])
        model.add_agent_of_interest("Agent1", ["Comp1"])

        response = dumps({"AgentOfInterest1" : model.get_agent_of_interest("Agent1")})
        assert response == dumps({"AgentOfInterest1" : bk_classes.AgentOfInterest(["Comp1"], False, False,  False, False, False, False, False, False, False, False)})
        
        # Name errors
        response = model.get_agent_of_interest("Agent5")
        assert response is None
        response = model.get_agent_of_interest("Agen@t5")
        assert response is None

    def test_remove_agent_of_interest(self):
        ''' Tests that the get of execute works '''
        model = bk.ModelEnv("agentofinteresttesting")
        model.add_component("Comp1", "This is component for add testing")
        model.add_component("Comp2", "This is component for add testing")
        model.add_agent("Agent1", "This is agent", ["Comp1", "Comp2"])
        model.add_agent_of_interest("Agent1", ["Comp1"])

        response = model.remove_agent_of_interest("Agent1")
        assert response == 0
        response = model.remove_agent_of_interest("Agent3")
        assert response == 1


class TestValidationMethods:
    ''' Tests for Validation methods '''
    def test_get_list_valid_agents(self):
        ''' Tests that the get list of valid agents of interest works '''
        model = bk.ModelEnv("validationtesting")
        model.add_component("Comp1", "This is component for add testing")
        model.add_component("Comp2", "This is component for add testing")
        model.add_component("Comp3", "This is component for add testing")
        model.add_component("Comp4", "This is component for add testing")
        model.add_component("Comp5", "This is component for add testing")
        model.add_component("Comp6", "This is component for add testing")
        model.add_agent("Agent1", "This is agent", ["Comp1", "Comp2", "Comp3", "Comp4", "Comp5", "Comp6"])
        model.add_agent("Agent2", "This is agent", ["Comp1", "Comp2", "Comp3", "Comp6"])
        model.add_agent("Agent3", "This is agent", ["Comp3", "Comp4", "Comp5", "Comp6"])
        model.add_agent("Agent4", "This is agent", ["Comp1", "Comp2", "Comp6"])
        model.add_agent("Agent5", "This is agent", ["Comp1", "Comp2", "Comp6"])
        model.add_agent_of_interest("Agent1", ["Comp1", "Comp2", "Comp3", "Comp4", "Comp5"], count = True)
        model.add_agent_of_interest("Agent2", ["Comp1", "Comp2", "Comp3"], count = True)
        model.add_agent_of_interest("Agent3", ["Comp3", "Comp4", "Comp5"], count = True)
        model.add_agent_of_interest("Agent4", ["Comp1", "Comp2", "Comp6"])

        response = model.get_list_valid_agents("Agent1", count = True)
        assert set(response) == set(["Agent2", "Agent3"])
        response = model.get_list_valid_agents("Agent1", "Comp3")
        assert set(response) == set(["Agent2", "Agent3"])
        response = model.get_list_valid_agents("Agent2", "Comp2")
        assert set(response) == set(["Agent1", "Agent4"])
        response = model.get_list_valid_agents("Agent4", "Comp6")
        assert set(response) == set([])

        # Name errors
        response = model.get_list_valid_agents("Agent5")
        assert set(response) == set()
        response = model.get_list_valid_agents("Agen@t5")
        assert set(response) == set()
        response = model.get_list_valid_agents("Agent1", "Comp 3")
        assert set(response) == set()
        response = model.get_list_valid_agents("Agent1", "Comp@3")
        assert set(response) == set()
        
    def test_get_list_valid_plot_data_types(self):
        ''' Tests that the get list of valid agents of interest works '''
        model = bk.ModelEnv("validationtesting")
        model.add_component("Comp1", "This is component for add testing")
        model.add_component("Comp2", "This is component for add testing")
        model.add_component("Comp3", "This is component for add testing")
        model.add_component("Comp4", "This is component for add testing")
        model.add_component("Comp5", "This is component for add testing")
        model.add_component("Comp6", "This is component for add testing")
        model.add_agent("Agent1", "This is agent", ["Comp1", "Comp2", "Comp3", "Comp4", "Comp5", "Comp6"])
        model.add_agent("Agent2", "This is agent", ["Comp1", "Comp2", "Comp3", "Comp6"])
        model.add_agent("Agent3", "This is agent", ["Comp3", "Comp4", "Comp5", "Comp6"])
        model.add_agent("Agent4", "This is agent", ["Comp1", "Comp2", "Comp6"])
        model.add_agent("Agent5", "This is agent", ["Comp1", "Comp2", "Comp3", "Comp4", "Comp5", "Comp6"])
        model.add_agent("Agent6", "This is agent", ["Comp1", "Comp2", "Comp3", "Comp6"])
        model.add_agent("Agent7", "This is agent", ["Comp3", "Comp4", "Comp5", "Comp6"])
        model.add_agent("Agent8", "This is agent", ["Comp1", "Comp2", "Comp6"])
        model.add_agent("Agent9", "This is agent", ["Comp1", "Comp2", "Comp6"])
        model.add_agent_of_interest("Agent1", ["Comp1", "Comp2", "Comp3", "Comp4", "Comp5"], count = True, mean_val=True, individual = True, max_val = True)
        model.add_agent_of_interest("Agent2", ["Comp1", "Comp2", "Comp3"], position = True, individual = True, max_val = True)
        model.add_agent_of_interest("Agent3", ["Comp3", "Comp4", "Comp5"], count = True)
        model.add_agent_of_interest("Agent4", ["Comp1", "Comp2", "Comp6"], individual = True, max_val = True)
        model.add_agent_of_interest("Agent5", ["Comp1", "Comp2", "Comp3", "Comp4", "Comp5"], count = True, mean_val=True, individual = True, max_val = True)
        model.add_agent_of_interest("Agent6", ["Comp1", "Comp2", "Comp3"], count = True, mean_val=True)
        model.add_agent_of_interest("Agent7", ["Comp3", "Comp4", "Comp5"], count = True)
        model.add_agent_of_interest("Agent8", ["Comp1", "Comp2", "Comp6"], individual = True, max_val = True, variance = True)
        
        response = model.get_list_valid_plot_data_types(["Agent1", "Agent3", "agent5", "Agent6", "Agent7"])
        assert set(response) == set(["count"])
        response = model.get_list_valid_plot_data_types(["Agent2", "agent4", "Agent8"])
        assert set(response) == set(["individual", "max_val"])
        response = model.get_list_valid_plot_data_types(["Agent1", "Agent5", "agent6"])
        assert set(response) == set(["count", "mean_val"])
        response = model.get_list_valid_plot_data_types(["Agent7"])
        assert set(response) == set(["count"])
        response = model.get_list_valid_plot_data_types(["Agent8"])
        assert set(response) == set(["individual", "max_val", "variance_val"])
        response = model.get_list_valid_plot_data_types(["Agent1", "Agent2", "Agent3", "Agent4", "Agent5", "Agent6", "Agent7", "Agent8"])
        assert set(response) == set([])

        # Name errors
        response = model.get_list_valid_plot_data_types(["Agent@7"])
        assert set(response) == set()
        response = model.get_list_valid_plot_data_types(["Agent1", "Agent3", "agent5", "Agent6", "Agent7@"])
        assert set(response) == set()
        response = model.get_list_valid_plot_data_types([])
        assert set(response) == set()


class TestFilingMethods:
    ''' Tests for Filing methods '''
    def test_create_model(self):
        ''' Tests that creating the model works '''
        model = bk.ModelEnv("filetesting")
        response = model.create_model("cool")
        assert response == 0
        response = model.create_model("cool.py")
        assert response == 0
        response = model.create_model("cool.py", True)
        assert response == 0


    def test_store_model(self):
        ''' Tests that creating the model works '''
        model = bk.ModelEnv("filetesting")
        response = model.store_model()
        assert response == 0
