"""
Testing for AGE-ABM Backend (backend, visual and utils)
@Author Max Hall
@Co-Authors Meghan Ireland and Matthew Fleischman

"""

import backend.backend_classes as bk_classes

class TestParentClass:
    ''' Tests for Component methods '''
    def test__init__(self):
        ''' Tests that the initialisation of the object '''
        object = bk_classes.ParentAgent("Hi I am description")
        assert str(type(object)) == "<class 'backend.backend_classes.ParentAgent'>"

        assert object.description == "Hi I am description"

    def test_edit_description(self):
        ''' Tests editing the description of the object '''
        object = bk_classes.ParentAgent("Hi I am description")

        object.edit_description("new_description")
        assert object.description == "new_description"

    def test_get_description(self):
        ''' Tests getting the description of the object '''
        object = bk_classes.ParentAgent("Hi I am description")

        response = object.get_description()
        assert response == "Hi I am description"


class TestImportDetailsClass:
    ''' Tests for Import Details methods '''
    def test__init__(self):
        ''' Tests that the initialisation of the object '''
        object1 = bk_classes.ImportDetails("nickname", "import_type")
        object2 = bk_classes.ImportDetails("nickname", "import_type", ["method1", "method2"])
        assert str(type(object1)) == "<class 'backend.backend_classes.ImportDetails'>"
        assert str(type(object2)) == "<class 'backend.backend_classes.ImportDetails'>"

        assert object1.nickname == "nickname"
        assert object1.import_type == "import_type"
        assert object1.list_of_methods is None

        assert object2.nickname == "nickname"
        assert object2.import_type == "import_type"
        assert set(object2.list_of_methods) == set(["method1", "method2"])


    def test_edit_nickname(self):
        ''' Tests editing the nickname of the object '''
        object1 = bk_classes.ImportDetails("nickname", "import_type")

        object1.edit_nickname("new_nickname")
        assert object1.nickname == "new_nickname"


    def test_get_nickname(self):
        ''' Tests getting the nickname of the object '''
        object1 = bk_classes.ImportDetails("nickname", "import_type")

        response = object1.get_nickname()
        assert response == "nickname"

    
    def test_edit_import_type(self):
        ''' Tests editing the import_type of the object '''
        object1 = bk_classes.ImportDetails("nickname", "import_type")

        object1.edit_import_type("new_import_type")
        assert object1.import_type == "new_import_type"


    def test_get_import_type(self):
        ''' Tests getting the import_type of the object '''
        object1 = bk_classes.ImportDetails("nickname", "import_type")

        response = object1.get_import_type()
        assert response == "import_type"

    def test_edit_list_of_methods(self):
        ''' Tests editing the list_of_methods of the object '''
        object1 = bk_classes.ImportDetails("nickname", "import_type")

        object1.edit_list_of_methods(["method1", "method2"])
        assert set(object1.list_of_methods) == set(["method1", "method2"])


    def test_get_list_of_methods(self):
        ''' Tests getting the list_of_methods of the object '''
        object1 = bk_classes.ImportDetails("nickname", "import_type")

        response = object1.get_list_of_methods()
        assert response is None 
        
        object1.edit_list_of_methods(["method1", "method2"])
        response = object1.get_list_of_methods()
        assert set(response) == set(["method1", "method2"])


class TestAttributeClass:
    ''' Tests for Attribute methods '''
    def test__init__(self):
        ''' Tests that the initialisation of the object '''
        object1 = bk_classes.Attribute("type_attr", "define_level")
        object2 = bk_classes.Attribute("type_attr", "define_level", 1)
        assert str(type(object1)) == "<class 'backend.backend_classes.Attribute'>"
        assert str(type(object2)) == "<class 'backend.backend_classes.Attribute'>"

        assert object1.type_attr == "type_attr"
        assert object1.define_level == "define_level"
        assert object1.default_value is None

        assert object2.type_attr == "type_attr"
        assert object2.define_level == "define_level"
        assert object2.default_value == 1


    def test_edit_description(self):
        ''' Tests editing the description of the object '''
        object1 = bk_classes.Attribute("type_attr", "define_level")

        object1.edit_description("new_description")
        assert object1.description == "new_description"


    def test_get_description(self):
        ''' Tests getting the description of the object '''
        object1 = bk_classes.Attribute("type_attr", "define_level")

        response = object1.get_description()
        assert response == ""


    def test_edit_type_attr(self):
        ''' Tests editing the type_attr of the object '''
        object1 = bk_classes.Attribute("type_attr", "define_level")

        object1.edit_type_attr("new_type_attr")
        assert object1.type_attr == "new_type_attr"


    def test_get_type_attr(self):
        ''' Tests getting the type_attr of the object '''
        object1 = bk_classes.Attribute("type_attr", "define_level")

        response = object1.get_type_attr()
        assert response == "type_attr"

    
    def test_edit_define_level(self):
        ''' Tests editing the define_level of the object '''
        object1 = bk_classes.Attribute("type_attr", "define_level")

        object1.edit_define_level("new_define_level")
        assert object1.define_level == "new_define_level"


    def test_get_define_level(self):
        ''' Tests getting the define_level of the object '''
        object1 = bk_classes.Attribute("type_attr", "define_level")

        response = object1.get_define_level()
        assert response == "define_level"


    def test_edit_default_value(self):
        ''' Tests editing the default_value of the object '''
        object1 = bk_classes.Attribute("type_attr", "define_level")

        object1.edit_default_value(1)
        assert object1.default_value == 1


    def test_get_default_value(self):
        ''' Tests getting the default_value of the object '''
        object1 = bk_classes.Attribute("type_attr", "define_level")

        response = object1.get_default_value()
        assert response is None 
        
        object1.edit_default_value(1)
        response = object1.get_default_value()
        assert response == 1


class TestComponentClass:
    ''' Tests for Component methods '''
    def test__init__(self):
        ''' Tests that the initialisation of the object '''
        object1 = bk_classes.Component("description", False, {})
        assert str(type(object1)) == "<class 'backend.backend_classes.Component'>"

        assert object1.description == "description"
        assert object1.is_class_component is False
        assert object1.attributes == {}


    def test_edit_description(self):
        ''' Tests editing the description of the object '''
        object1 = bk_classes.Component("description", False, {})

        object1.edit_description("new_description")
        assert object1.description == "new_description"


    def test_get_description(self):
        ''' Tests getting the description of the object '''
        object1 = bk_classes.Component("description", False, {})

        response = object1.get_description()
        assert response == "description"

    
    def test_edit_is_class_component(self):
        ''' Tests editing the is_class_component boolean of the object '''
        object1 = bk_classes.Component("description", False, {})

        object1.edit_is_class_component(True)
        assert object1.is_class_component is True


    def test_get_is_class_component(self):
        ''' Tests getting the is_class_component boolean of the object '''
        object1 = bk_classes.Component("description", False, {})

        response = object1.get_is_class_component()
        assert response is False


    def test_edit_attributes(self):
        ''' Tests editing the attributes of the object '''
        object1 = bk_classes.Component("description", False, {})

        object1.edit_attributes({"newAttribute" : None})
        assert object1.attributes == {"newAttribute" : None}


    def test_get_attributes(self):
        ''' Tests getting the attributes of the object '''
        object1 = bk_classes.Component("description", False, {})

        response = object1.get_attributes()
        assert response == {}
        
        object1.edit_attributes({"newAttribute" : None})
        response = object1.get_attributes()
        assert response == {"newAttribute" : None}


class TestAgentClass:
    ''' Tests for Agent methods '''
    def test__init__(self):
        ''' Tests that the initialisation of the object '''
        object1 = bk_classes.Agent("description", ["Comp1", "Comp2"])
        assert str(type(object1)) == "<class 'backend.backend_classes.Agent'>"

        assert object1.description == "description"
        assert set(object1.given_components) == set(["Comp1", "Comp2"])

        x_coord = 1
        y_coord = 2
        object2 = bk_classes.Agent("description", ["Comp1", "Comp2"], [x_coord, y_coord])
        assert str(type(object2)) == "<class 'backend.backend_classes.Agent'>"

        assert object2.description == "description"
        assert set(object1.given_components) == set(["Comp1", "Comp2"])
        assert set(object2.init_coordinates) == set([1, 2])


    def test_edit_description(self):
        ''' Tests editing the description of the object '''
        object1 = bk_classes.Agent("description", ["Comp1", "Comp2"])

        object1.edit_description("new_description")
        assert object1.description == "new_description"


    def test_get_description(self):
        ''' Tests getting the description of the object '''
        object1 = bk_classes.Agent("description", ["Comp1", "Comp2"])

        response = object1.get_description()
        assert response == "description"


    def test_edit_given_components(self):
        ''' Tests editing the given_components of the object '''
        object1 = bk_classes.Agent("description", ["Comp1", "Comp2"])

        object1.edit_given_components(["Comp1", "Comp2"])
        assert set(object1.given_components) == set(["Comp1", "Comp2"])


    def test_get_given_components(self):
        ''' Tests getting the given_components of the object '''
        object1 = bk_classes.Agent("description", ["Comp1", "Comp2"])
        
        response = object1.get_given_components()
        assert set(response) == set(["Comp1", "Comp2"])


    def test_edit_init_coordinates(self):
        ''' Tests editing the init_coordinates of the object '''
        object1 = bk_classes.Agent("description", ["Comp1", "Comp2"])
        
        x_coord = 1
        y_coord = 2
        object1.edit_init_coordinates([x_coord, y_coord])
        assert set(object1.init_coordinates) == set([1,2])


    def test_get_init_coordinates(self):
        ''' Tests getting the init_coordinates of the object '''
        object1 = bk_classes.Agent("description", ["Comp1", "Comp2"])

        response = object1.get_init_coordinates()
        assert response is None 
        
        x_coord = 1
        y_coord = 2
        object1.edit_init_coordinates([x_coord, y_coord])
        response = object1.get_init_coordinates()
        assert set(response) == set([1,2])


class TestSystemClass:
    ''' Tests for System methods '''
    def test__init__(self):
        ''' Tests that the initialisation of the object '''
        object1 = bk_classes.System("description")
        assert str(type(object1)) == "<class 'backend.backend_classes.System'>"

        assert object1.description == "description"
        assert object1.execute_function_body == "\tdef execute(self):\n\t\t# Do not edit above this line!\n\t\t# Code below and indent correctly using tabs, not spaces!\n\t\tpass"
        assert object1.start == None
        assert object1.end == None
        assert object1.frequency == None
        assert object1.generators == {}

        start = 1
        end = 200
        frequency = 10
        object2 = bk_classes.System("description", "execute_function_body", start, end, frequency, {"generator1" : None})
        assert str(type(object2)) == "<class 'backend.backend_classes.System'>"

        assert object2.description == "description"
        assert object2.execute_function_body == "execute_function_body"
        assert object2.start == 1
        assert object2.end == 200
        assert object2.frequency == 10
        assert object2.generators == {"generator1" : None}


    def test_edit_description(self):
        ''' Tests editing the description of the object '''
        object1 = bk_classes.System("description")

        object1.edit_description("new_description")
        assert object1.description == "new_description"


    def test_get_description(self):
        ''' Tests getting the description of the object '''
        object1 = bk_classes.System("description")

        response = object1.get_description()
        assert response == "description"


    def test_edit_execute_function_body(self):
        ''' Tests editing the execute_function_body of the object '''
        object1 = bk_classes.System("description")

        object1.edit_execute_function_body("new_execute_function_body")
        assert object1.execute_function_body == "new_execute_function_body"


    def test_get_execute_function_body(self):
        ''' Tests getting the execute_function_body of the object '''
        object1 = bk_classes.System("description", "execute_function_body")
        
        response = object1.get_execute_function_body()
        assert response == "execute_function_body"


    def test_edit_start(self):
        ''' Tests editing the start of the object '''
        object1 = bk_classes.System("description")
        
        object1.edit_start(1)
        assert object1.start == 1


    def test_get_start(self):
        ''' Tests getting the start of the object '''
        object1 = bk_classes.System("description")

        response = object1.get_start()
        assert response is None 
        
        object1.edit_start(1)
        response = object1.get_start()
        assert response == 1


    def test_remove_start(self):
        ''' Tests removing the start of the object '''
        object1 = bk_classes.System("description")
        object1.edit_start(1)

        object1.remove_start()
        assert object1.start is None


    def test_edit_end(self):
        ''' Tests editing the end of the object '''
        object1 = bk_classes.System("description")
        
        object1.edit_end(200)
        assert object1.end == 200


    def test_get_end(self):
        ''' Tests getting the end of the object '''
        object1 = bk_classes.System("description")

        response = object1.get_end()
        assert response is None 
        
        object1.edit_end(200)
        response = object1.get_end()
        assert response == 200


    def test_remove_end(self):
        ''' Tests removing the end of the object '''
        object1 = bk_classes.System("description")
        object1.edit_end(200)

        object1.remove_end()
        assert object1.end is None


    def test_edit_frequency(self):
        ''' Tests editing the frequency of the object '''
        object1 = bk_classes.System("description")
        
        object1.edit_frequency(20)
        assert object1.frequency == 20


    def test_get_frequency(self):
        ''' Tests getting the frequency of the object '''
        object1 = bk_classes.System("description")

        response = object1.get_frequency()
        assert response is None 
        
        object1.edit_frequency(20)
        response = object1.get_frequency()
        assert response == 20


    def test_remove_frequency(self):
        ''' Tests removing the frequency of the object '''
        object1 = bk_classes.System("description")
        object1.edit_frequency(2)

        object1.remove_frequency()
        assert object1.frequency is None


    def test_edit_generators(self):
        ''' Tests editing the generators of the object '''
        object1 = bk_classes.System("description")

        object1.edit_generators({"generator1" : None})
        assert object1.generators == {"generator1" : None}


    def test_get_generators(self):
        ''' Tests getting the generators of the object '''
        object1 = bk_classes.System("description")

        response = object1.get_generators()
        assert response == {}

        object1.edit_generators({"generator1" : None})
        response = object1.get_generators()
        assert response == {"generator1" : None}


class TestGeneratorClass:
    ''' Tests for Generator methods '''
    def test__init__(self):
        ''' Tests that the initialisation of the object '''
        object1 = bk_classes.Generator("description", "nickname", "return_value", {})
        assert str(type(object1)) == "<class 'backend.backend_classes.Generator'>"

        assert object1.description == "description"
        assert object1.nickname == "nickname"
        assert object1.return_value == "return_value"
        assert object1.attributes == {}


    def test_edit_description(self):
        ''' Tests editing the description of the object '''
        object1 = bk_classes.Generator("description", "nickname", "return_value", {})

        object1.edit_description("new_description")
        assert object1.description == "new_description"


    def test_get_description(self):
        ''' Tests getting the description of the object '''
        object1 = bk_classes.Generator("description", "nickname", "return_value", {})

        response = object1.get_description()
        assert response == "description"


    def test_edit_nickname(self):
        ''' Tests editing the nickname of the object '''
        object1 = bk_classes.Generator("description", "nickname", "return_value", {})

        object1.edit_nickname("new_nickname")
        assert object1.nickname == "new_nickname"


    def test_get_nickname(self):
        ''' Tests getting the nickname of the object '''
        object1 = bk_classes.Generator("description", "nickname", "return_value", {})
        
        response = object1.get_nickname()
        assert response == "nickname"


    def test_edit_return_value(self):
        ''' Tests editing the return_value of the object '''
        object1 = bk_classes.Generator("description", "nickname", "return_value", {})
        
        object1.edit_return_value("new_return_value")
        assert object1.return_value == "new_return_value"


    def test_get_return_value(self):
        ''' Tests getting the return_value of the object '''
        object1 = bk_classes.Generator("description", "nickname", "return_value", {})

        response = object1.get_return_value()
        assert response == "return_value"

    def test_edit_attributes(self):
        ''' Tests editing the attributes of the object '''
        object1 = bk_classes.Generator("description", "nickname", "return_value", {})

        object1.edit_attributes({"attribute1" : None})
        assert object1.attributes == {"attribute1" : None}


    def test_get_attributes(self):
        ''' Tests getting the attributes of the object '''
        object1 = bk_classes.Generator("description", "nickname", "return_value", {})

        response = object1.get_attributes()
        assert response == {}

        object1.edit_attributes({"attribute1" : None})
        response = object1.get_attributes()
        assert response == {"attribute1" : None}


class TestAgentOfInterestClass:
    ''' Tests for Agent Of Interest methods '''
    def test__init__(self):
        ''' Tests that the initialisation of the object '''
        object1 = bk_classes.AgentOfInterest(["Comp1", "Comp2"], False, False, False, False, False, False, False, False, False, False)
        assert str(type(object1)) == "<class 'backend.backend_classes.AgentOfInterest'>"

        assert set(object1.comp_of_interest) == set(["Comp1", "Comp2"])
        assert object1.count == False
        assert object1.position == False
        assert object1.individual == False
        assert object1.mean_val == False
        assert object1.median_val == False
        assert object1.total_val == False
        assert object1.max_val == False
        assert object1.min_val == False
        assert object1.variance == False
        assert object1.std_dev == False

    def test_edit_comp_of_interest(self):
        ''' Tests editing the comp_of_interest of the object '''
        object1 = bk_classes.AgentOfInterest(["Comp1", "Comp2"], False, False, False, False, False, False, False, False, False, False)

        object1.edit_comp_of_interest(["Comp1", "Comp2", "Comp3"])
        assert set(object1.comp_of_interest) == set(["Comp1", "Comp2", "Comp3"])


    def test_get_comp_of_interest(self):
        ''' Tests getting the comp_of_interest of the object '''
        object1 = bk_classes.AgentOfInterest(["Comp1", "Comp2"], False, False, False, False, False, False, False, False, False, False)

        assert set(object1.get_comp_of_interest()) == set(["Comp1", "Comp2"])

    def test_edit_count(self):
        ''' Tests editing the count boolean of the object '''
        object1 = bk_classes.AgentOfInterest(["Comp1", "Comp2"], False, False, False, False, False, False, False, False, False, False)

        object1.edit_count(True)
        assert object1.count is True


    def test_get_count(self):
        ''' Tests getting the count boolean of the object '''
        object1 = bk_classes.AgentOfInterest(["Comp1", "Comp2"], False, False, False, False, False, False, False, False, False, False)

        response = object1.get_count()
        assert response == False

    def test_edit_position(self):
        ''' Tests editing the position boolean of the object '''
        object1 = bk_classes.AgentOfInterest(["Comp1", "Comp2"], False, False, False, False, False, False, False, False, False, False)

        object1.edit_position(True)
        assert object1.position is True


    def test_get_position(self):
        ''' Tests getting the position boolean of the object '''
        object1 = bk_classes.AgentOfInterest(["Comp1", "Comp2"], False, False, False, False, False, False, False, False, False, False)

        response = object1.get_position()
        assert response == False

    def test_edit_individual(self):
        ''' Tests editing the individual boolean of the object '''
        object1 = bk_classes.AgentOfInterest(["Comp1", "Comp2"], False, False, False, False, False, False, False, False, False, False)

        object1.edit_individual(True)
        assert object1.individual is True


    def test_get_individual(self):
        ''' Tests getting the individual boolean of the object '''
        object1 = bk_classes.AgentOfInterest(["Comp1", "Comp2"], False, False, False, False, False, False, False, False, False, False)

        response = object1.get_individual()
        assert response == False

    def test_edit_mean_val(self):
        ''' Tests editing the mean_val boolean of the object '''
        object1 = bk_classes.AgentOfInterest(["Comp1", "Comp2"], False, False, False, False, False, False, False, False, False, False)

        object1.edit_mean_val(True)
        assert object1.mean_val is True


    def test_get_mean_val(self):
        ''' Tests getting the mean_val boolean of the object '''
        object1 = bk_classes.AgentOfInterest(["Comp1", "Comp2"], False, False, False, False, False, False, False, False, False, False)

        response = object1.get_mean_val()
        assert response == False

    def test_edit_median_val(self):
        ''' Tests editing the median_val boolean of the object '''
        object1 = bk_classes.AgentOfInterest(["Comp1", "Comp2"], False, False, False, False, False, False, False, False, False, False)

        object1.edit_median_val(True)
        assert object1.median_val is True


    def test_get_median_val(self):
        ''' Tests getting the median_val boolean of the object '''
        object1 = bk_classes.AgentOfInterest(["Comp1", "Comp2"], False, False, False, False, False, False, False, False, False, False)

        response = object1.get_median_val()
        assert response == False

    def test_edit_total_val(self):
        ''' Tests editing the total_val boolean of the object '''
        object1 = bk_classes.AgentOfInterest(["Comp1", "Comp2"], False, False, False, False, False, False, False, False, False, False)

        object1.edit_total_val(True)
        assert object1.total_val is True


    def test_get_total_val(self):
        ''' Tests getting the total_val boolean of the object '''
        object1 = bk_classes.AgentOfInterest(["Comp1", "Comp2"], False, False, False, False, False, False, False, False, False, False)

        response = object1.get_total_val()
        assert response == False

    def test_edit_max_val(self):
        ''' Tests editing the max_val boolean of the object '''
        object1 = bk_classes.AgentOfInterest(["Comp1", "Comp2"], False, False, False, False, False, False, False, False, False, False)

        object1.edit_max_val(True)
        assert object1.max_val is True


    def test_get_max_val(self):
        ''' Tests getting the max_val boolean of the object '''
        object1 = bk_classes.AgentOfInterest(["Comp1", "Comp2"], False, False, False, False, False, False, False, False, False, False)

        response = object1.get_max_val()
        assert response == False

    def test_edit_min_val(self):
        ''' Tests editing the min_val boolean of the object '''
        object1 = bk_classes.AgentOfInterest(["Comp1", "Comp2"], False, False, False, False, False, False, False, False, False, False)

        object1.edit_min_val(True)
        assert object1.min_val is True


    def test_get_min_val(self):
        ''' Tests getting the min_val boolean of the object '''
        object1 = bk_classes.AgentOfInterest(["Comp1", "Comp2"], False, False, False, False, False, False, False, False, False, False)

        response = object1.get_min_val()
        assert response == False

    def test_edit_variance(self):
        ''' Tests editing the variance boolean of the object '''
        object1 = bk_classes.AgentOfInterest(["Comp1", "Comp2"], False, False, False, False, False, False, False, False, False, False)

        object1.edit_variance(True)
        assert object1.variance is True


    def test_get_variance(self):
        ''' Tests getting the variance boolean of the object '''
        object1 = bk_classes.AgentOfInterest(["Comp1", "Comp2"], False, False, False, False, False, False, False, False, False, False)

        response = object1.get_variance()
        assert response == False

    def test_edit_std_dev(self):
        ''' Tests editing the std_dev boolean of the object '''
        object1 = bk_classes.AgentOfInterest(["Comp1", "Comp2"], False, False, False, False, False, False, False, False, False, False)

        object1.edit_std_dev(True)
        assert object1.std_dev is True


    def test_get_std_dev(self):
        ''' Tests getting the std_dev boolean of the object '''
        object1 = bk_classes.AgentOfInterest(["Comp1", "Comp2"], False, False, False, False, False, False, False, False, False, False)

        response = object1.get_std_dev()
        assert response == False


class TestModelDetailsClass:
    ''' Tests for Model Details methods '''
    def test__init__(self):
        ''' Tests that the initialisation of the object '''
        object1 = bk_classes.ModelDetails("modelname", "description")
        assert str(type(object1)) == "<class 'backend.backend_classes.ModelDetails'>"

        assert object1.name == "modelname"
        assert object1.description == "description"
        assert object1.initial_parameters == {"num_agents" : {}}
        assert object1.gridworld == False
        assert object1.sim_length == 1
        assert object1.sim_seed is None

        object2 = bk_classes.ModelDetails("modelname", "description", {}, True, 20, 44)
        assert str(type(object2)) == "<class 'backend.backend_classes.ModelDetails'>"

        assert object2.name == "modelname"
        assert object2.description == "description"
        assert object2.initial_parameters == {}
        assert object2.gridworld == True
        assert object2.sim_length == 20
        assert object2.sim_seed == 44

    
    def test_edit_name(self):
        ''' Tests editing the name of the object '''
        object1 = bk_classes.ModelDetails("modelname", "description")

        object1.edit_name("new_name")
        assert object1.name == "new_name"


    def test_get_name(self):
        ''' Tests getting the name of the object '''
        object1 = bk_classes.ModelDetails("modelname", "description")

        response = object1.get_name()
        assert response == "modelname"


    def test_edit_description(self):
        ''' Tests editing the description of the object '''
        object1 = bk_classes.ModelDetails("modelname", "description")

        object1.edit_description("new_description")
        assert object1.description == "new_description"


    def test_get_description(self):
        ''' Tests getting the description of the object '''
        object1 = bk_classes.ModelDetails("modelname", "description")

        response = object1.get_description()
        assert response == "description"


    def test_add_initial_parameter(self):
        ''' Tests adding the initial_parameter of the object '''
        object1 = bk_classes.ModelDetails("modelname", "description")

        response = object1.add_initial_parameter("initial_parameter_name_1", "initial_parameter_type")
        assert response == 0

        response = object1.add_initial_parameter("initial_parameter_name_2", "initial_parameter_type", 5)
        assert response == 0

        response = object1.add_initial_parameter("initial_parameter_name_2", "initial_parameter_type")
        assert response == 1
    

    def test_edit_initial_parameter(self):
        ''' Tests editing the initial_parameters of the object '''
        object1 = bk_classes.ModelDetails("modelname", "description")
        object1.add_initial_parameter("initial_parameter_name_1", "initial_parameter_type")

        response =  object1.edit_initial_parameter("initial_parameter_name_1", initial_parameter_value = 1)
        assert response == 0
        
        response =  object1.edit_initial_parameter("initial_parameter_name_1", initial_parameter_type = "initial_parameter_type_2")
        assert response == 0

        response = object1.edit_initial_parameter("initial_parameter_name_2", initial_parameter_type = "initial_parameter_type_2", initial_parameter_value = 1)
        assert response == 1


    def test_get_initial_parameter(self):
        ''' Tests getting the initial_parameters of the object '''
        object1 = bk_classes.ModelDetails("modelname", "description")
        object1.add_initial_parameter("initial_parameter_name_1", "initial_parameter_type")

        response = object1.get_initial_parameter("initial_parameter_name_1")
        assert set(response) == set(["initial_parameter_type", None])

        response = object1.get_initial_parameter("initial_parameter_name_2")
        assert response == 1


    def test_remove_initial_parameter(self):
        ''' Tests getting the initial_parameters of the object '''
        object1 = bk_classes.ModelDetails("modelname", "description")
        object1.add_initial_parameter("initial_parameter_name_1", "initial_parameter_type")

        response = object1.remove_initial_parameter("initial_parameter_name_1")
        assert response == 0
        
        response = object1.remove_initial_parameter("initial_parameter_name_1")
        assert response == 1


    def test_get_list_initial_parameters(self):
        ''' Tests getting the initial_parameters of the object '''
        object1 = bk_classes.ModelDetails("modelname", "description")
        object1.add_initial_parameter("initial_parameter_name_1", "initial_parameter_type")
        object1.add_initial_parameter("initial_parameter_name_2", "initial_parameter_type")
        object1.add_initial_parameter("initial_parameter_name_3", "initial_parameter_type")
        object1.add_initial_parameter("initial_parameter_name_4", "initial_parameter_type")

        response = object1.get_list_initial_parameters()
        assert set(response) == set(["initial_parameter_name_1", "initial_parameter_name_2", "initial_parameter_name_3", "initial_parameter_name_4"])

    
    def test_add_num_agent(self):
        ''' Tests adding the num_agent of the object '''
        object1 = bk_classes.ModelDetails("modelname", "description")

        response = object1.add_num_agent("num_agent_1")
        assert response == 0

        response = object1.add_num_agent("num_agent_2", 20)
        assert response == 0

        response = object1.add_num_agent("num_agent_2")
        assert response == 1
    

    def test_edit_num_agent(self):
        ''' Tests editing the num_agent of the object '''
        object1 = bk_classes.ModelDetails("modelname", "description")
        object1.add_num_agent("num_agent_1")

        response =  object1.edit_num_agent("num_agent_1", init_num = 20)
        assert response == 0

        response = object1.edit_num_agent("num_agent_2", init_num = 20)
        assert response == 1


    def test_get_num_agent(self):
        ''' Tests getting the num_agent of the object '''
        object1 = bk_classes.ModelDetails("modelname", "description")
        object1.add_num_agent("num_agent_1")

        response = object1.get_num_agent("num_agent_1")
        assert response is None

        response = object1.get_num_agent("num_agent_2")
        assert response == 1


    def test_remove_num_agent(self):
        ''' Tests getting the num_agent of the object '''
        object1 = bk_classes.ModelDetails("modelname", "description")
        object1.add_num_agent("num_agent_1")

        response = object1.remove_num_agent("num_agent_1")
        assert response == 0
        
        response = object1.remove_num_agent("num_agent_1")
        assert response == 1


    def test_get_list_num_agent(self):
        ''' Tests getting the num_agent of the object '''
        object1 = bk_classes.ModelDetails("modelname", "description")
        object1.add_num_agent("num_agent_1")
        object1.add_num_agent("num_agent_2")
        object1.add_num_agent("num_agent_3", 30)
        object1.add_num_agent("num_agent_4", 20)

        response = object1.get_list_num_agents()
        assert set(response) == set(["num_agent_1", "num_agent_2", "num_agent_3", "num_agent_4"])


    def test_edit_sim_length(self):
        ''' Tests editing the sim_length of the object '''
        object1 = bk_classes.ModelDetails("modelname", "description")

        object1.edit_sim_length(20)
        assert object1.sim_length == 20


    def test_get_sim_length(self):
        ''' Tests getting the sim_length of the object '''
        object1 = bk_classes.ModelDetails("modelname", "description")

        response = object1.get_sim_length()
        assert response == 1

    
    def test_edit_sim_seed(self):
        ''' Tests editing the sim_seed of the object '''
        object1 = bk_classes.ModelDetails("modelname", "description")

        object1.edit_sim_seed(44)
        assert object1.sim_seed == 44


    def test_get_sim_seed(self):
        ''' Tests getting the sim_seed of the object '''
        object1 = bk_classes.ModelDetails("modelname", "description")

        response = object1.get_sim_seed()
        assert response is None

        object1.edit_sim_seed(44)
        response = object1.get_sim_seed()
        assert response == 44
        