"""
Testing for AGE-ABM Backend (backend, visual and utils)
@Author Max Hall
@Co-Authors Meghan Ireland and Matthew Fleischman

"""

''' I make use of jsonpickle for testing get as it breaks the class into a string to allow for comparison '''
from jsonpickle import dumps
import backend.backend as bk
import visuals.visual as vis
import visuals.visualutils as vis_utils


class TestInitialisation:
    ''' Ensures that the class is of the correct type and initialises correctly '''

    def test__init__(self):
        ''' Ensures that the class is of the correct type and initialises correctly '''
        model = bk.ModelEnv("tesingvisutils")
        vismodel = vis.VisualEnv(model)
        writingfile = vis_utils.WriteVisualFiles(model, vismodel)
        assert str(type(writingfile)) == "<class 'visuals.visualutils.WriteVisualFiles'>"

        assert dumps(writingfile.model) == dumps(bk.ModelEnv("tesingvisutils"))
        assert dumps(writingfile.visualiser) == dumps(vis.VisualEnv(model))
        writingfile.visualisation_section = ""


class TestCreatingCodeSections:
    ''' Ensures code to be written to files is generated properly '''
    def test_create_visualisation_file_section(self):
        ''' Tests that the adding of visualisation section works '''
        model = bk.ModelEnv("tesingvisutils")
        vismodel = vis.VisualEnv(model)
        writingfile = vis_utils.WriteVisualFiles(model, vismodel)
        model.add_component("Comp1", "This is component for add testing")
        model.add_component("Comp2", "This is component for add testing")
        model.add_attribute("Comp1", "Attr1", "int", "comp_lvl", 1)
        model.add_attribute("Comp1", "Attr2", "bool", "comp_lvl", True)
        model.add_agent("Agent1", "This is agent", ["Comp1", "Comp2"])
        model.add_agent("Agent2", "This is agent", ["Comp1", "Comp2"])
        model.add_agent("Agent3", "This is agent", ["Comp1"])
        model.add_agent_of_interest("Agent1", ["Comp1", "Comp2"], count = True, mean_val=True, individual = True, max_val = True)
        model.add_agent_of_interest("Agent2", ["Comp1", "Comp2"], count = True, mean_val=True, individual = True, position = True, max_val = True)
        model.add_agent_of_interest("Agent3", ["Comp1"], count = True, mean_val=True, individual = True, max_val = True)
        vismodel.add_agent_visualisation("Plot1", ["Agent1", "Agent2", "Agent3"], "mean_val", "plot", "Comp1", "Attr1")
        vismodel.add_agent_visualisation("Plot2", ["Agent1", "Agent2"], "individual", "plot", "Comp1", "Attr1")
        vismodel.add_agent_visualisation("Plot3", ["Agent1", "Agent2", "Agent3"], "count", "plot")

        response = writingfile.create_visualisation_file_section("Plot1")
        assert response == 0

        response = writingfile.create_visualisation_file_section("Plot2")
        assert response == 0

        response = writingfile.create_visualisation_file_section("Plot4")
        assert response == 1


class TestWritingToFiles:
    ''' Ensures code to be written to files is generated properly '''
    def test_create_visualisation_file(self):
        ''' Tests that creating the visual file works '''
        model = bk.ModelEnv("tesingvisutils")
        vismodel = vis.VisualEnv(model)
        writingfile = vis_utils.WriteVisualFiles(model, vismodel)
        model.add_component("Comp1", "This is component for add testing")
        model.add_component("Comp2", "This is component for add testing")
        model.add_attribute("Comp1", "Attr1", "int", "comp_lvl", 1)
        model.add_attribute("Comp1", "Attr2", "bool", "comp_lvl", True)
        model.add_agent("Agent1", "This is agent", ["Comp1", "Comp2"])
        model.add_agent("Agent2", "This is agent", ["Comp1", "Comp2"])
        model.add_agent("Agent3", "This is agent", ["Comp1"])
        model.add_agent_of_interest("Agent1", ["Comp1", "Comp2"], count = True, mean_val=True, individual = True, max_val = True)
        model.add_agent_of_interest("Agent2", ["Comp1", "Comp2"], count = True, mean_val=True, position = True, max_val = True)
        model.add_agent_of_interest("Agent3", ["Comp1"], count = True, mean_val=True, individual = True, max_val = True)
        vismodel.add_agent_visualisation("Plot1", ["Agent1", "Agent2", "Agent3"], "mean_val", "plot", "Comp1", "Attr1")
        vismodel.add_agent_visualisation("Plot2", ["Agent1", "Agent2"], "mean_val", "plot", "Comp1", "Attr1")
        vismodel.add_agent_visualisation("Plot3", ["Agent1", "Agent2", "Agent3"], "count", "plot")
        writingfile.create_visualisation_file_section("Plot1")

        response = writingfile.create_visualisation_file("Plot1")
        assert response == 0
        response = writingfile.create_visualisation_file("Plot2")
        assert response == 0
        response = writingfile.create_visualisation_file("Plot3")
        assert response == 0

        # Name errors
        response = writingfile.create_visualisation_file("Plot @#3")
        assert response == 1
        response = writingfile.create_visualisation_file("Plot4")
        assert response == 1
        
        # Below can be run if you modify visualutils so that the visualisingFiles folder is never created to test that the except works
        #response = writingfile.create_visualisation_file("Plot1")
        #assert response == 1
