"""
Backend Classes for AGE-ABM Visualising
@Author Max Hall
@Co-Authors Meghan Ireland and Matthew Fleischman

"""

class VisualiseAgent:
    """ This class is for agent visualisation purposes and stores 
    details about an agent to visualise its desired plots """

    def __init__(self, visual_agent_name : str, compare_agent_list : list[str], plot_data_type : str
                    , plot_type : str, comp_of_interest : str, attr_of_interest : str):
        self.visual_agent_name = visual_agent_name
        self.compare_agent_list = compare_agent_list.copy()
        self.comp_of_interest = comp_of_interest
        self.attr_of_interest = attr_of_interest
        self.plot_data_type = plot_data_type
        self.plot_type = plot_type

    def get_visual_agent_name(self):
        """Gets the name of the agent to visualise.

        Parameters
        ----------

        Returns
        -------
        visual_agent_name : str
            | Returns name of the agent to return.
        """
        return self.visual_agent_name

    def get_compare_agent_list(self):
        """Gets the list of agents to be compared to the current visual agent.

        Parameters
        ----------

        Returns
        -------
        compare_agent_list : list[str]
            | Returns list of agent names to compare to be returned.
        """
        return self.compare_agent_list.copy()

    def get_comp_of_interest(self):
        """Gets the str of the comp_of_interest.

        Parameters
        ----------

        Returns
        -------
        comp_of_interest : str
            | Returns comp_of_interest string to be returned.
        """
        return self.comp_of_interest

    def get_attr_of_interest(self):
        """Gets the str of the attr_of_interest.

        Parameters
        ----------

        Returns
        -------
        attr_of_interest : str
            
            | Returns attr_of_interest string to be returned.
        """
        return self.attr_of_interest

    def get_plot_data_type(self):
        """Gets the str of the plot_data_type.

        Parameters
        ----------

        Returns
        -------
        plot_data_type : str
            | Returns plot_data_type string to be returned.
        """
        return self.plot_data_type

    def get_plot_type(self):
        """Gets the str of the plot_type.

        Parameters
        ----------

        Returns
        -------
        plot_type : str
            | Returns plot_type string to be returned.
        """
        return self.plot_type
