# CSC3003S - AGE-ABM Capstone - FLSMAT002, HLLMAX005, IRLMEG002

## Overview:

Welcome to ECApp, 

This is the readme for ECApp, a UI designed to allow a low/no code environment for anyone using the ECAgent python library who is struggling to grasp the deeper core coding techniques or simpily wants to speed up their coding process for models being built using the ECAgent library.

This readme explains how to install the required libraries, run the model in a virtual environment, run the tests for the model, run a coverage report and lastly clean up the directory once you're done for the day.

It also explains where one can find the files which have been generated or created throughout running the application. These files can be run standalone with different parameters and the argparse library has been used to allow this.

Inside of **JSONDemos/** we have provided some demo models which you can load into the model to add to or experiment with. 
* Money2d.json - This is a JSON of a Gridworld version of the baisic tutorial provided by Brandon, the maker of ECAgent
* Money2d.json - This is a JSON of the advanced tutorial provided by Brandon Gower-Winter, the maker of ECAgent

Git Repository: https://gitlab.cs.uct.ac.za/capstoneirl_hll_fls/capstone


## Explaining the Structure:
* As mentioned above, the backend is completely removed from the visual interface.
* On call of **make run** which executes ecapp.py:
  * The user interacts soely with the classes inside **ECApp/interface**.
  * The ModelEnv class inside of **ECApp/backend/backend.py** is what acts as the database object.
    * This stores individual data related to imports, components, attributes etc. in respective dictionaries as objects.
    * These objects are of types from **ECApp/backend/backend_classes.py**.
    * **ECApp/backend/backendutils.py** is used by the backend for generating files.
  * When a plot is created inside of ECApp the classes inside of **ECApp/visuals** are used to create and plot the data.
  * The visualisations are dependent on the model created and thus a ModelEnv object is parsed as a parameter.

## Key Running Tips:

* The following tips will be provided:
  * How to install the required libraries.
  * How to run the model in a virtual environment.
  * How to run the unit tests for the backend of the model.
  * How to generate a coverage report for the backend of the model.
  * How to clean up the directory after use.
  * This will also tell you how to install the required libraries and run the model if you don't wish to use a virtual environment.

### Installing the required libraries:

* Move into the ECAgent Folder, then run **make install** in the terminal.
* This will also initialise your virtual environment.

### Running the model in a virtual environment:

* Move into the ECAgent Folder, then run **make run** in the terminal.

### Running the unit tests for the backend of the model:

* Move into the ECAgent Folder, then run **make test** in the terminal.

### Generating a coverage report for the backend of the model:

* Move into the ECAgent Folder, then run **make test-coverage** in the terminal.

### Cleaning up the directory after use:

* Move into the ECAgent Folder, then run **make clean** in the terminal.

### Without penv:

* Move into the ECAgent Folder, then run **pip3 install -Ur requirements.txt** in the terminal.
* Then run **python ecapp.py** in the terminal.
* Cleaning up the directory will work the same as described above.


## Finding Your Files:

* If you compiled your model you can find it inside **ECApp/runtimeFiles/**.
* If you saved your model you can find the JSON inside **ECApp/ModelAsJSONFiles/**.
  * Do not modify this file or it will not work when loading it back in.
  * If you want the visualising files your model used you can find them inside **ECApp/runtimeFiles/**.


## Additional Information:

* All files have been verified for code quality using the pylint quality guidelines
  * All files achieve a score above 8
  * The only places where penalisation occured were areas that did not make sense to amend or the code required it to be as such

### Pylint Scores

* Table for Backend Files:

| Name of File    | Pylint Score |
| --------------- | ------------ |
| Backend         | 9.02         |
| Backendutils    | 8.28         |
| Backend_classes | 9.39         |
| visual          | 9.52         |
| visualutils     | 9.43         |
| visual_classes  | 9.50         |
| helper_methods  | 9.92         |

* Table for Interface Files:

| Name of File    | Pylint Score |
| --------------- | ------------ |
| main            | 9.88         |
| home            | 10.00        |
| libraries       | 9.61         |
| components      | 9.58         |
| agents          | 9.70         |
| systems         | 8.50         |
| edit_attributes | 9.47         |
| execute         | 9.16         |
| data_collection | 9.50         |
| model_details   | 9.87         |
| compile         | 10.00        |
| visualisation   | 8.07         |
